bl_info = {
	"name": "Export Model",
	"description": "Export model",
	"author": "Xavier de Boysson",
	"version": (1, 1),
	"blender": (2, 6, 3),
	"location": "File > Export",
	"warning": "",
	"wiki_url": "",
	"tracker_url": "",
	"category": "Import-Export"}



import bpy
import re
from bpy.props import *
import mathutils, math, struct
import os
from os import remove
import time
import shutil
from PIL import Image, ImageDraw
from random import randint

def writeString(file, string):
	file.write(bytes(string, 'UTF-8'))

def do_export(context, props, filepath, pngimg):

	i = 0
	
	mat_x90 = mathutils.Matrix.Rotation(-math.pi/2, 4, 'X')
	ob = context.selectable_objects
	file = open(filepath, 'wb')
	tab = '    '
	indices = []
	
	me = bpy.context.object.data
	for poly in me.polygons:
		for loop_index in poly.loop_indices:
			indices.append(me.loops[loop_index].vertex_index)
	
	i=0
	
	im = None
	for objs in ob:
		
		nbVertices = 0
		if (objs.type == 'MESH'):
			mesh = objs.to_mesh(context.scene, props.apply_modifiers, 'PREVIEW')
			if props.world_space:
				mesh.transform(objs.matrix_world)
			if props.rot_x90:
				mesh.transform(mat_x90)			

			for face in mesh.tessfaces:
				for index in face.vertices:
					nbVertices+=1

			imgSize = int(math.ceil(math.sqrt(3*(nbVertices*4)+(nbVertices*3)+2)))
			size = (imgSize,imgSize)
			width, height = size
			im = Image.new('RGB', size, (255, 255, 255))
			
			draw = ImageDraw.Draw(im).point
			
			
		#	Vertices
			x = 0
			y = 0
			
			writeString(file, '\r\nExporting mesh with %i vertices in image of size %ix%i\r\n' % (nbVertices, imgSize, imgSize))

# HEADER START
	#Nb Vertices
			a1, b1, c1, d1 = struct.unpack('BBBB', struct.pack('f', nbVertices))
			draw((x, y), (int(a1), int(b1), int(c1)))
			x+=1
			draw((x, y), (int(d1), 255, 255))
			x+=1
# HEADER END


			#color = objs.active_material.diffuse_color
			basename = objs.name.capitalize()
			#	Number of vertices
		#	file.write(struct.pack('<i', (len(mesh.tessfaces))))




##########
#VERTICES#
##########
			cnt = 0
			for faceid, face in enumerate(mesh.tessfaces):
				for i, index in enumerate(face.vertices):
					vert = mesh.vertices[index]
					a1, b1, c1, d1 = struct.unpack('BBBB', struct.pack('f', vert.co.x))
					a2, b2, c2, d2 = struct.unpack('BBBB', struct.pack('f', vert.co.y))
					a3, b3, c3, d3 = struct.unpack('BBBB', struct.pack('f', vert.co.z))

					draw((x, y), (int(a1), int(b1), int(c1)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(d1), int(a2), int(b2)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(c2), int(d2), int(a3)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(b3), int(c3), int(d3)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1
					cnt+=1
			writeString(file, '\r\nExported %i vertices\r\n' % cnt)
			
#########
#NORMALS#
#########
			cnt = 0		
			for faceid, face in enumerate(mesh.tessfaces):
				for i, index in enumerate(face.vertices):
					vert = mesh.vertices[index]
					a1, b1, c1, d1 = struct.unpack('BBBB', struct.pack('f', vert.normal.x))
					a2, b2, c2, d2 = struct.unpack('BBBB', struct.pack('f', vert.normal.y))
					a3, b3, c3, d3 = struct.unpack('BBBB', struct.pack('f', vert.normal.z))

					draw((x, y), (int(a1), int(b1), int(c1)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(d1), int(a2), int(b2)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(c2), int(d2), int(a3)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1

					draw((x, y), (int(b3), int(c3), int(d3)))
					x+=1
					if (x>width-1):
						x = 0
						y+=1
					cnt+=1
			writeString(file, 'Exported %i normals\r\n' % cnt)
########
#COLORS#
########
			cnt = 0
			if props.export_colors and mesh.tessface_vertex_colors:
				active_col_layer = mesh.tessface_vertex_colors.active
				active_col_layer = active_col_layer.data
				for faceid, face in enumerate(mesh.tessfaces):
					col = active_col_layer[faceid]
					col = col.color1[:], col.color2[:], col.color3[:], col.color4[:]
					for i, index in enumerate(face.vertices):
						a1, b1, c1, d1 = struct.unpack('BBBB', struct.pack('f', col[i][0]))
						a2, b2, c2, d2 = struct.unpack('BBBB', struct.pack('f', col[i][1]))
						a3, b3, c3, d3 = struct.unpack('BBBB', struct.pack('f', col[i][2]))

						draw((x, y), (int(a1), int(b1), int(c1)))
						x+=1
						if (x>width-1):
							x = 0
							y+=1

						draw((x, y), (int(d1), int(a2), int(b2)))
						x+=1
						if (x>width-1):
							x = 0
							y+=1

						draw((x, y), (int(c2), int(d2), int(a3)))
						x+=1
						if (x>width-1):
							x = 0
							y+=1

						draw((x, y), (int(b3), int(c3), int(d3)))
						x+=1
						if (x>width-1):
							x = 0
							y+=1
						cnt+=1
			writeString(file, 'Exported %i colors\r\n' % cnt)


#TEXTURE COORDS
			cnt = 0
			if props.export_coords and len(mesh.uv_textures) > 0:
				uv_layer = mesh.tessface_uv_textures.active
				#for face in mesh.tessfaces:
				for faceid, face in enumerate(mesh.tessfaces):
					faceUV = uv_layer.data[face.index]
					for i, index in enumerate(face.vertices):
					# for index in face.vertices:
						a1, b1, c1, d1 = struct.unpack('BBBB', struct.pack('f', faceUV.uv[i][0]))
						a2, b2, c2, d2 = struct.unpack('BBBB', struct.pack('f', faceUV.uv[i][1]))
						
						draw((x, y), (int(a1), int(b1), int(c1)))

						x+=1
						if (x>width-1):
							x = 0
							y+=1

						draw((x, y), (int(d1), int(a2), int(b2)))

						x+=1
						if (x>width-1):
							x = 0
							y+=1
						
						draw((x, y), (int(c2), int(d2), 255))

						x+=1
						if (x>width-1):
							x = 0
							y+=1
							
#						i+=1
						cnt +=1

			writeString(file, 'Exported %i texture coords\r\n' % cnt)

	del draw
	im.save(pngimg, "PNG")

	file.flush()
	file.close()

	return True


###### EXPORT OPERATOR #######
class Export_mdl(bpy.types.Operator):
	'''Exports the active Object as mdl file.'''
	bl_idname = "export_object.mdl"
	bl_label = "Export to map file"
	filepath = StringProperty(subtype='FILE_PATH')
	
	apply_modifiers = BoolProperty(name="Apply Modifiers",
							description="Applies the Modifiers",
							default=True)

	rot_x90 = BoolProperty(name="Convert to Y-up",
							description="Rotate 90 degrees around X to convert to y-up",
							default=True)
	
	world_space = BoolProperty(name="Export into Worldspace",
							description="Transform the Vertexcoordinates into Worldspace",
							default=True)

	export_coords = BoolProperty(name="Export texture coordinates",
							description="Exports vertex texture coordinates",
							default=True)

	export_colors = BoolProperty(name="Export vertex colors",
							description="Exports vertex colors",
							default=True)
							
	@classmethod
	def poll(cls, context):
		return context.active_object.type in ['MESH', 'CURVE', 'SURFACE', 'FONT']

	def execute(self, context):
		start_time = time.time()
		print('\r\n_____START_____')
		Filepath = bpy.path.ensure_ext(self.filepath, ".log")
		pngImg = bpy.path.ensure_ext(self.filepath, ".png")
		exported = do_export(context, self.properties, Filepath, pngImg)
		
		if exported:
			print('finished export in %s seconds' %((time.time() - start_time)))
			
		return {'FINISHED'}

	def invoke(self, context, event):
		wm = context.window_manager

		if True:
			# File selector
			wm.fileselect_add(self) # will run self.execute()
			return {'RUNNING_MODAL'}
		elif True:
			# search the enum
			wm.invoke_search_popup(self)
			return {'RUNNING_MODAL'}
		elif False:
			# Redo popup
			return wm.invoke_props_popup(self, event) #
		elif False:
			return self.execute(context)


### REGISTER ###

def menu_func(self, context):
	self.layout.operator(Export_mdl.bl_idname, text="Export Model")

def register():
	bpy.utils.register_module(__name__)

	bpy.types.INFO_MT_file_export.append(menu_func)

def unregister():
	bpy.utils.unregister_module(__name__)

	bpy.types.INFO_MT_file_export.remove(menu_func)

if __name__ == "__main__":
	register()